<?php

namespace IQBall\Api\Controller;

use IQBall\App\Control;
use IQBall\Core\Data\Account;
use IQBall\Core\Http\HttpCodes;
use IQBall\Core\Http\HttpRequest;
use IQBall\Core\Http\HttpResponse;
use IQBall\Core\Http\JsonHttpResponse;
use IQBall\Core\Model\TacticModel;
use IQBall\Core\Model\TeamModel;
use IQBall\Core\Validation\FieldValidationFail;
use IQBall\Core\Validation\Validators;

/**
 * API endpoint related to tactics
 */
class APITacticController {
    private TacticModel $tacticModel;
    private TeamModel $teamModel;

    /**
     * @param TacticModel $tacticModel
     * @param TeamModel $teamModel
     */
    public function __construct(TacticModel $tacticModel, TeamModel $teamModel) {
        $this->tacticModel = $tacticModel;
        $this->teamModel = $teamModel;
    }

    /**
     * update name of tactic, specified by tactic identifier, given in url.
     * @param int $tactic_id
     * @param Account $account
     * @return HttpResponse
     */
    public function updateName(int $tactic_id, Account $account): HttpResponse {
        return Control::runChecked([
            "name" => [Validators::lenBetween(1, 50), Validators::nameWithSpaces()],
        ], function (HttpRequest $request) use ($tactic_id, $account) {

            $failures = $this->tacticModel->updateName($tactic_id, $request["name"], $account->getUser()->getId());

            if (!empty($failures)) {
                //TODO find a system to handle Unauthorized error codes more easily from failures.
                return new JsonHttpResponse($failures, HttpCodes::BAD_REQUEST);
            }

            return HttpResponse::fromCode(HttpCodes::OK);
        });
    }

    /**
     * @param int $id
     * @param Account $account
     * @return HttpResponse
     */
    public function saveContent(int $id, Account $account): HttpResponse {
        return Control::runChecked([
            "content" => [],
        ], function (HttpRequest $req) use ($id) {
            if ($fail = $this->tacticModel->updateContent($id, json_encode($req["content"]))) {
                return new JsonHttpResponse([$fail], HttpCodes::BAD_REQUEST);
            }
            return HttpResponse::fromCode(HttpCodes::OK);
        });
    }


    public function canShareTactic(int $tacticId, Account $account): HttpResponse {
        if ($this->tacticModel->canShareTactic($tacticId, $account)) {
            return HttpResponse::fromCode(HttpCodes::OK);
        }
        return new JsonHttpResponse(["message" => "Vous ne pouvez pas partager cette tactique"], HttpCodes::FORBIDDEN);
    }


    public function canShareTacticToTeam(int $tacticId, Account $account): HttpResponse {
        return Control::runChecked([
            "id" => [],
            "name" => [],
            "picture" => [],
            "main_color" => [],
            "second_color" => []
        ], function (HttpRequest $request) use ($tacticId, $account) {
            if ($this->canShareTactic($tacticId, $account)) {
                if ($this->teamModel->canShareTacticToTeam($request["id"], $account->getUser()->getEmail())) {
                    return HttpResponse::fromCode(HttpCodes::OK);
                }
            }
            return new JsonHttpResponse(["message" => "Action non autorisée"], HttpCodes::FORBIDDEN);
        });
    }


    public function shareTacticToTeam(int $tacticId, Account $account): HttpResponse {
        return Control::runChecked([
            "id" => [],
            "name" => [],
            "picture" => [],
            "main_color" => [],
            "second_color" => []
        ], function (HttpRequest $request) use ($tacticId, $account) {
            $this->teamModel->shareTacticToTeam($request["id"], $tacticId);
            return HttpResponse::fromCode(HttpCodes::OK);
        });
    }

    public function shareTacticToAccount(int $tacticId, Account $account): HttpResponse {
        return Control::runChecked([
            "email" => [],
        ], function (HttpRequest $request) use ($tacticId, $account) {
            $this->tacticModel->shareTacticToAccountMail($request["email"], $tacticId);
            return HttpResponse::fromCode(HttpCodes::OK);
        });
    }

    public function unshareTacticToTeam(int $tacticId, Account $account): HttpResponse {
        return Control::runChecked([
            "id" => [],
            "name" => [],
            "picture" => [],
            "mainColor" => [],
            "secondColor" => []
        ], function (HttpRequest $request) use ($tacticId, $account) {
            if ($this->teamModel->canShareTacticToTeam($request["id"], $account->getUser()->getEmail())) {
                $this->teamModel->unshareTacticToTeam($tacticId, $request["id"]);
                return HttpResponse::fromCode(HttpCodes::OK);
            }
            return new JsonHttpResponse(["message" => "Action non autorisée"], HttpCodes::FORBIDDEN);
        });
    }
}

