<?php

namespace IQBall\App\Controller;

use IQBall\App\Session\SessionHandle;
use IQBall\App\Validator\TacticValidator;
use IQBall\App\ViewHttpResponse;
use IQBall\Core\Data\CourtType;
use IQBall\Core\Data\TacticInfo;
use IQBall\Core\Http\HttpCodes;
use IQBall\Core\Model\TacticModel;
use IQBall\Core\Validation\ValidationFail;

class EditorController {
    private TacticModel $model;

    public function __construct(TacticModel $model) {
        $this->model = $model;
    }

    /**
     * @param TacticInfo $tactic
     * @return ViewHttpResponse the editor view for given tactic
     */
    private function openEditorFor(TacticInfo $tactic): ViewHttpResponse {
        return ViewHttpResponse::react("views/Editor.tsx", [
            "id" => $tactic->getId(),
            "name" => $tactic->getName(),
            "content" => $tactic->getContent(),
            "courtType" => $tactic->getCourtType()->name(),
        ]);
    }

    public function createNew(): ViewHttpResponse {
        return ViewHttpResponse::react("views/NewTacticPanel.tsx", []);
    }

    /**
     * @return ViewHttpResponse the editor view for a test tactic.
     */
    private function openTestEditor(CourtType $courtType): ViewHttpResponse {
        return ViewHttpResponse::react("views/Editor.tsx", [
            "id" => -1, //-1 id means that the editor will not support saves
            "name" => TacticModel::TACTIC_DEFAULT_NAME,
            "content" => '{"players": [], "objects": [], "actions": []}',
            "courtType" => $courtType->name(),
        ]);
    }

    /**
     * creates a new empty tactic, with default name
     * If the given session does not contain a connected account,
     * open a test editor.
     * @param SessionHandle $session
     * @param CourtType $type
     * @return ViewHttpResponse the editor view
     */
    public function createNewOfKind(CourtType $type, SessionHandle $session): ViewHttpResponse {

        $action = $session->getAccount();

        if ($action == null) {
            return $this->openTestEditor($type);
        }

        $tactic = $this->model->makeNewDefault($session->getAccount()->getUser()->getId(), $type);
        return $this->openEditorFor($tactic);
    }

    /**
     * returns an editor view for a given tactic
     * @param int $id the targeted tactic identifier
     * @param SessionHandle $session
     * @return ViewHttpResponse
     */
    public function openEditor(int $id, SessionHandle $session): ViewHttpResponse {
        $tactic = $this->model->get($id);

        $ownerId = $session->getAccount()->getUser()->getId();

        $ids = $this->model->getOwnerIdTacticShared($id, $session->getAccount()->getUser()->getId());

        if(isset($ids)) {
            $ownerId = $ids['owner'];
        }

        $failure = TacticValidator::validateAccess($id, $tactic, $ownerId);

        if ($failure != null) {
            return ViewHttpResponse::twig('error.html.twig', ['failures' => [$failure]], HttpCodes::NOT_FOUND);
        }

        return $this->openEditorFor($tactic);
    }
}
