<?php

namespace IQBall\Core\Model;

use IQBall\Core\Data\Account;
use IQBall\Core\Data\CourtType;
use IQBall\App\Session\SessionHandle;
use IQBall\Core\Data\TacticInfo;
use IQBall\Core\Gateway\AccountGateway;
use IQBall\Core\Gateway\TacticInfoGateway;
use IQBall\Core\Validation\ValidationFail;

class TacticModel {
    public const TACTIC_DEFAULT_NAME = "Nouvelle tactique";

    private TacticInfoGateway $tactics;
    private AccountGateway $users;

    /**
     * @param TacticInfoGateway $tactics
     */
    public function __construct(TacticInfoGateway $tactics, AccountGateway $users) {
        $this->tactics = $tactics;
        $this->users = $users;
    }

    /**
     * creates a new empty tactic, with given name
     * @param string $name
     * @param int $ownerId
     * @param CourtType $type
     * @return TacticInfo
     */
    public function makeNew(string $name, int $ownerId, CourtType $type): TacticInfo {
        $id = $this->tactics->insert($name, $ownerId, $type);
        return $this->tactics->get($id);
    }

    /**
     * creates a new empty tactic, with a default name
     * @param int $ownerId
     * @param CourtType $type
     * @return TacticInfo|null
     */
    public function makeNewDefault(int $ownerId, CourtType $type): ?TacticInfo {
        return $this->makeNew(self::TACTIC_DEFAULT_NAME, $ownerId, $type);
    }

    /**
     * Tries to retrieve information about a tactic
     * @param int $id tactic identifier
     * @return TacticInfo|null or null if the identifier did not match a tactic
     */
    public function get(int $id): ?TacticInfo {
        return $this->tactics->get($id);
    }

    /**
     * Return the nb last tactics created
     *
     * @param integer $nb
     * @return array<array<string,mixed>>
     */

    /**
     * Return the nb last tactics
     *
     * @param integer $nb
     * @param integer $ownerId
     * @return array<array<string,mixed>>
     */
    public function getLast(int $nb, int $ownerId): array {
        return $this->tactics->getLastOwnedBy($nb, $ownerId);
    }

    /**
     * Get all the tactics of the owner
     *
     * @param integer $ownerId
     * @return array|null
     */
    public function getAll(int $ownerId): ?array {
        return $this->tactics->getAllOwnedBy($ownerId);
    }

    public function getAllTacticSharedTeam(int $ownerId) : ?array
    {
        return $this->tactics->getAllTacticSharedTeam($ownerId);
    }

    public function getAllTacticSharedAccount(int $ownerId) : ?array {
        return $this->tactics->getAllTacticSharedAccount($ownerId);
    }

    public function getAllTacticShared(int $ownerId) : ?array {
        $allTactics = [];
        $allTacticsSharedTeam = $this->tactics->getAllTacticSharedTeam($ownerId);
        if(isset($allTacticsSharedTeam)) {
            foreach ($allTacticsSharedTeam as $tactic) {
                if(!in_array($tactic, $allTactics)) {
                    array_push($allTactics, $tactic);
                }
            }
        }

        $allTacticsSharedAccount = $this->tactics->getAllTacticSharedAccount($ownerId);
        if(isset($allTacticsSharedAccount)) {
            foreach ($allTacticsSharedAccount as $tactic) {
                if(!in_array($tactic, $allTactics)) {
                    array_push($allTactics, $tactic);
                }
            }
        }
        return $allTactics;
    }

    public function getAllTacticSharedOwned(int $ownerId) : ?array {
        $allTactics = [];
        $allTacticsSharedTeamOwned = $this->tactics->getAllTacticSharedTeamOwned($ownerId);
        if(isset($allTacticsSharedTeamOwned)) {
            foreach ($allTacticsSharedTeamOwned as $tactic) {
                if(!in_array($tactic, $allTactics)) {
                    array_push($allTactics, $tactic);
                }
            }
        }

        $allTacticsSharedAccountOwned = $this->tactics->getAllTacticSharedAccountOwned($ownerId);
        if(isset($allTacticsSharedAccountOwned)) {
            foreach ($allTacticsSharedAccountOwned as $tactic) {
                if(!in_array($tactic, $allTactics)) {
                    array_push($allTactics, $tactic);
                }
            }
        }
        return $allTactics;
    }

    public function shareTacticToAccount(int $accountId, int $tacticId): int
    {
        return $this->users->shareTacticToAccount($accountId, $tacticId);
    }

    public function shareTacticToAccountMail(string $email, int $tacticId): ?int
    {
        $account = $this->users->getAccountFromMail($email);
        var_dump($account);
        $accountId = $account->getUser()->getId();
        var_dump($accountId);
        if(isset($accountId)) {
            return $this->shareTacticToAccount($accountId, $tacticId);
        }
        return null;
    }


    public function unshareTactic(int $tacticId): int
    {
        return $this->users->unshareTactic($tacticId);
    }

    public function unshareTacticToAccount(int $tacticId, int $accountId): int
    {
        return $this->users->unshareTacticToAccount($tacticId, $accountId);
    }

    public function getOwnerIdTacticShared(int $tactidId, int $accountId) : ?array {
        $ids = [];
        $idT = $this->tactics->getOwnerIdTacticSharedTeam($tactidId, $accountId);
        if(isset($idT)) {
            foreach ($idT as $id) {
                if(!in_array($id, $ids)) {
                    array_push($ids, $id);
                }
            }
        }

        $idA = $this->tactics->getOwnerIdTacticSharedAccount($tactidId, $accountId);
        if(isset($idA)) {
            foreach ($idA as $id) {
                if(!in_array($id, $ids)) {
                    array_push($ids, $id);
                }
            }
        }

        if($ids != null) {
            return $ids[0];
        }
        return null;
    }


    /**
     * Update the name of a tactic
     * @param int $id the tactic identifier
     * @param string $name the new name to set
     * @return ValidationFail[] failures, if any
     */
    public function updateName(int $id, string $name, int $authId): array {

        $tactic = $this->tactics->get($id);

        if ($tactic == null) {
            return [ValidationFail::notFound("Could not find tactic")];
        }

        if ($tactic->getOwnerId() != $authId) {
            return [ValidationFail::unauthorized()];
        }

        if (!$this->tactics->updateName($id, $name)) {
            return [ValidationFail::error("Could not update name")];
        }
        return [];
    }

    public function updateContent(int $id, string $json): ?ValidationFail {
        if (!$this->tactics->updateContent($id, $json)) {
            return ValidationFail::error("Could not update content");
        }
        return null;
    }

    public function canShareTactic(int $id, Account $account) : bool {
        if($this->get($id)->getOwnerId() != $account->getUser()->getId()) {
            return false;
        }
        return true;
    }
}
